@@ -1,22 +1,25 @@
-;This code was initially based on
-;dictionary.el written by Torsten Hilbrich <Torsten.Hilbrich@gmx.net>
-;but now probably doesn't contain original code.
-;Most of the code has been written
-;from scratch by Aleksey Cheusov <vle@gmx.net>, 2004
+; dictem.el - DICT protocol client (rfc-2229) for [X]Emacs
+
+; This code was initially based on
+; dictionary.el written by Torsten Hilbrich <Torsten.Hilbrich@gmx.net>
+; but now probably doesn't contain original code.
+; Most of the code has been written
+; from scratch by Aleksey Cheusov <vle@gmx.net>, 2004.
 ;
-;DictEm is free software; you can redistribute it and/or modify
-;it under the terms of the GNU General Public License as published by
-;the Free Software Foundation; either version 2 of the License, or
-;(at your option) any later version.
+; DictEm is free software; you can redistribute it and/or modify it
+; under the terms of the GNU General Public License as published by
+; the Free Software Foundation; either version 2 of the License, or
+; (at your option) any later version.
 ;
-;DictEm is distributed in the hope that it will be useful,
-;but WITHOUT ANY WARRANTY; without even the implied warranty of
-;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;GNU General Public License for more details.
+; DictEm is distributed in the hope that it will be useful, but
+; WITHOUT ANY WARRANTY; without even the implied warranty of
+; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+; General Public License for more details.
 ;
-;You should have received a copy of the GNU General Public License
-;along with this program; if not, write to the Free Software
-;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+; You should have received a copy of the GNU General Public License
+; along with this program; if not, write to the Free Software
+; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+; 02111-1307, USA
 
 (require 'cl)
 
@@ -52,12 +55,12 @@ dict-1.9.14 or later (or compatible) is recomented."
   :group 'dictem
   :type 'string)
 
-(defcustom dictem-default-strategy "."
+(defcustom dictem-default-strategy nil
   "The default search strategy."
   :group 'dictem
   :group 'string)
 
-(defcustom dictem-default-database "*"
+(defcustom dictem-default-database nil
   "The default database name."
   :group 'dictem
   :group 'string)
@@ -94,6 +97,12 @@ Instead, existing buffer will be erased and used to show results.
   :group 'dictem
   :type 'boolean)
 
+(defcustom dictem-empty-initial-input
+  nil
+  "If `t' the `dictem-read-query' leave initial input empty"
+  :group 'dictem
+  :type 'boolean)
+
 ;;;;;            Faces             ;;;;;
 
 (defface dictem-reference-definition-face
@@ -323,7 +332,7 @@ by functions run from dictem-postprocess-each-definition-hook.")
 	  (get-buffer-create dictem-temp-buffer-name) nil
 	  "-P" "-" "-m"
 	  "-d" (if database database "*")
-	  "-s" (if strategy strategy dictem-default-strategy)
+	  "-s" (if strategy strategy ".")
 	  "-h" (if server server dictem-server)
 	  "-p" (dictem-get-port port)
 	  "--client" (dictem-client-text)
@@ -420,6 +429,33 @@ and returns alist containing database names and descriptions"
 	err))
      )))
 
+(defun dictem-get-default-strategy (&optional def-strat)
+  "Gets the default search strategy"
+  (if def-strat
+      def-strat
+    (if dictem-default-strategy
+	dictem-default-strategy
+      (if dictem-last-strategy
+	  dictem-last-strategy
+	"."))))
+
+(defun dictem-get-default-database (&optional def-db)
+  "Gets the default database"
+  (defun dictem-extract-dbname (database)
+    (cond
+     ((consp database) (dictem-extract-dbname (car database)))
+     ((stringp database) database)
+     (t (error "The database should be either stringp or consp"))
+     ))
+
+  (if def-db
+      (dictem-extract-dbname def-db)
+    (if dictem-default-database
+	(dictem-extract-dbname dictem-default-database)
+      (if dictem-last-database
+	  (dictem-extract-dbname dictem-last-database)
+	"*"))))
+
 ;;;;;      Low Level Functions     ;;;;;
 
 (defun dictem-get-line ()
@@ -456,18 +492,12 @@ and returns alist containing database names and descriptions"
       ((completion-ignore-case t)
        (str (completing-read
 	     (concat prompt " [" default "]: ")
-	     alist
-	     nil
-	     t
-	     nil
-	     history
-	     default))
+	     alist nil t nil history default))
        (str-cons (assoc str alist)))
-;    str-cons))
     (cond
-     ((and str-cons (cdr str-cons))
-      (cdr str-cons))
-     ((and str-cons (null (cdr str-cons)))
+     ((and str-cons (consp str-cons) (cdr str-cons))
+      str-cons)
+     ((and str-cons (consp str-cons))
       (car str-cons))
      (t nil))))
 
@@ -549,11 +579,7 @@ to enter a search strategy."
    "strategy"
    (dictem-prepand-special-strats
     (dictem-remove-value-from-alist dictem-strategy-alist))
-   (if default-strat
-       default-strat
-     (if dictem-strategy-history
-	 (car dictem-strategy-history)
-       dictem-default-strategy))
+   (dictem-get-default-strategy default-strat)
    'dictem-strategy-history))
 
 (defun dictem-select-database (spec-dbs user-dbs &optional default-db)
@@ -572,11 +598,7 @@ to enter a database name."
     (dictem-select
      "db"
      (if spec-dbs (dictem-prepand-special-dbs dbs2) dbs2)
-     (if default-db
-	 default-db
-       (if dictem-database-history
-	   (car dictem-database-history)
-	 "*"))
+     (dictem-get-default-database default-db)
      'dictem-database-history)))
 
 (defun dictem-read-query (&optional default-query)
@@ -588,7 +610,8 @@ to enter a database name."
        nil 'dictem-query-history default-query)
     (read-string
      (concat "query [" default-query "]: ")
-     nil 'dictem-query-history default-query t)))
+     (if dictem-empty-initial-input nil default-query)
+     'dictem-query-history default-query t)))
 
 
 ;;;;;;;;    Search Functions     ;;;;;;;
@@ -641,7 +664,7 @@ to enter a database name."
      ((stringp databases)
       (dictem-local-call-dict-internal-iter fun (cons databases nil)))
      ((consp databases)
-      (dictem-local-call-dict-internal-iter fun databases))
+      (dictem-local-call-dict-internal-iter fun (cdr databases)))
      (t (error "wrong type of argument"))
      )
 
@@ -731,9 +754,10 @@ to enter a database name."
 			     (buffer-substring-no-properties beg (point)))
 			    dictem-error-messages)))
 		 (kill-region beg (point))))
-	  (setq dictem-last-database database)
 	  ex_status))))
 
+  (setq dictem-last-database database)
+  (setq dictem-last-strategy strategy)
   (dictem-call-dict-internal 'dictem-local-run-dict-search databases)))
 
 (defun dictem-base-define (databases query strategy)
@@ -781,9 +805,9 @@ to enter a database name."
 			     (buffer-substring-no-properties beg (point)))
 			    dictem-error-messages)))
 		 (kill-region beg (point))))
-	  (setq dictem-last-database database)
 	  ex_status))))
 
+  (setq dictem-last-database database)
   (dictem-call-dict-internal 'dictem-local-run-dict-define databases)))
 
 (defun dictem-base-match (databases query strategy)
@@ -825,9 +849,10 @@ to enter a database name."
 			     (buffer-substring-no-properties beg (point)))
 			    dictem-error-messages)))
 		 (kill-region beg (point))))
-	  (setq dictem-last-database database)
 	  ex_status))))
 
+  (setq dictem-last-database database)
+  (setq dictem-last-strategy strategy)
   (dictem-call-dict-internal 'dictem-local-run-dict-match databases)))
 
 (defun dictem-base-show-info (databases b c)
@@ -861,9 +886,9 @@ to enter a database name."
 			 (buffer-substring-no-properties beg (point)))
 			dictem-error-messages)))
 	     (kill-region beg (point))))
-      (setq dictem-last-database database)
       ex_status))
 
+  (setq dictem-last-database database)
   (dictem-call-dict-internal 'run-dict-show-info databases)))
 
 (defun dictem-base-show-server (a b c)
@@ -966,8 +991,8 @@ to enter a database name."
 	    (dictem-ensure-buffer)
 	  (dictem))
 ;	(set-buffer-file-coding-system coding-system)
-	(make-local-variable 'dictem-last-strategy)
-	(make-local-variable 'dictem-last-database)
+	(make-local-variable 'dictem-default-strategy)
+	(make-local-variable 'dictem-default-database)
 	(make-local-variable 'case-replace)
 	(make-local-variable 'case-fold-search)
 
@@ -981,8 +1006,6 @@ to enter a database name."
 	(set (make-local-variable 'dictem-use-existing-buffer) use-existing-buf)
 
 	;;;;;;;;;;;;;;
-	(setq dictem-last-strategy strategy)
-	(setq dictem-last-database database)
 	(setq case-replace nil)
 	(setq case-fold-search nil)
 	(setq dictem-error-messages nil)
@@ -1182,7 +1205,7 @@ and returns a list containing protocol, server, port and path on nil if fails"
 (define-key dictem-mode-map "\177" 'scroll-down)
 
 ; Define on click
-(if (featurep `xemacs)
+(if (featurep 'xemacs)
     (define-key dictem-mode-map [button2]
       'dictem-define-on-click)
   (define-key dictem-mode-map [mouse-2]
@@ -1251,8 +1274,8 @@ shows matches in it."
   (interactive
    (list 
     (dictem-read-query (thing-at-point 'word))
-    (dictem-select-database t t dictem-last-database)
-    (dictem-select-strategy)))
+    (dictem-select-database t t (dictem-get-default-database))
+    (dictem-select-strategy (dictem-get-default-strategy))))
   (dictem-run 'dictem-base-match database query strat))
 
 (defun dictem-run-define (query database)
@@ -1262,7 +1285,7 @@ shows definitions in it."
   (interactive
    (list
     (dictem-read-query (thing-at-point 'word))
-    (dictem-select-database t t dictem-last-database)))
+    (dictem-select-database t t (dictem-get-default-database))))
   (dictem-run 'dictem-base-define database query nil))
 
 (defun dictem-run-search (query database strat)
@@ -1272,15 +1295,18 @@ shows definitions in it."
   (interactive
    (list
     (dictem-read-query (thing-at-point 'word))
-    (dictem-select-database t t dictem-last-database)
-    (dictem-select-strategy)))
+    (dictem-select-database t t (dictem-get-default-database))
+    (dictem-select-strategy (dictem-get-default-strategy))))
   (dictem-run 'dictem-base-search database query strat))
 
 (defun dictem-run-show-info (database)
   "Asks a user about database name
 creates new *dictem* buffer and
 shows information about it."
-  (interactive (list (dictem-select-database nil nil dictem-last-database)))
+  (interactive (list
+		(dictem-select-database
+		 nil nil
+		 (dictem-get-default-database))))
   (dictem-run 'dictem-base-show-info database))
 
 (defun dictem-run-show-server ()
